/*****************************************************
    FrSky DHT 8
:  (cdrr.mechanic@gmail.com)
http://luckytech.ru/frsky_telemetry.html
Chip type               : Atmega48PA
AVR Core Clock frequency: 8,000000 MHz
*****************************************************/

#include <mega48pa.h>
#include <stdio.h>
#include <stdlib.h>   
#include <delay.h>

#define LED_delay 150
#define delay_sound_start 4

#define s_metr 123  // { 
#define s_bat 38  // & 
#define s_alt 94  // ^ 
#define s_lipo_off 85  // U 
#define s_onboard_volt 86  // V 
#define s_lipo 87  // W 
#define s_bat_low 126  // ~ 
#define s_percent 37  // % 


#define Dig1 PORTD.7
#define Dig2 PORTD.6
#define Dig3 PORTD.5

#define Btn  PINC.1
#define Buz  PORTC.0

#define Led1 PORTD.2
#define Led2 PORTD.3
#define Led3 PORTD.4


// Declare your global variables here
unsigned char Symbol[12] = {
0b00100010, //0
0b11111010, //1
0b01100100, //2
0b01101000, //3
0b10111000, //4
0b00101001, //5
0b00100001, //6
0b01111010, //7
0b00100000, //8
0b00101000, //9
0b11111111, //blank
0b11111101  //-
};


char alarm_count=0;
char delay_count=0;

char time0=0;

char time_sound=0; //    
char Byte2Play[4]; //     UART (  )
char ByteCount=0; //     UART.     4 ...
char ByteCountTimer=0; //      
bit PlayOn=0; //   -    Byte2Play  1

char count_metr=0;


char disp=1; //1 - AV1, 2-AV2, 3 - BAT, 4-Alt

unsigned char btn_push=0; //    -     ,     :(   -   1,   
unsigned char delayADC=0; //    -      ,      

unsigned char AV1=0;
unsigned char AV2=0;


unsigned int Altitude=0;
unsigned int PrevAltitude=0;
unsigned char PrevVoltage=0;
int BAT=0;

char ReceiveBuffer[31];
char ReceivePos0=0; // Index of first byte to remove from queue.
char ReceivePos1=0; // Index where to add next byte.
char COM_DataPresent() { return ReceivePos0!=ReceivePos1; }

/*
http://www.helifreak.com/showthread.php?t=333661
4.20v = 100%
4.03v = 76%
3.86v = 52%
3.83v = 42%
3.79v = 30%
3.70v = 11%
3.6?v = 0%

4.00V--84%
3.96---77%
3.93---70%
3.90---63%
3.86---56%
3.83---48%
3.80---43%
3.76---35%
3.73---27%
3.70---21%
3.67---14%

This one is very pessimistic. The others have 50% at about 3.85v. This has 50% at 3.95v

100% 4.20 volts
90% 4.15
80% 4.10
70% 4.05
60% 4.00
50% 3.95
40% 3.90
30% 3.85
20% 3.80
10% 3.70
*/
 
char Volt2Persent(int Voltage)
{
    if ((Voltage>=415)) return 100;
    if ((Voltage>=410)&&(Voltage<415)) return 90;
    if ((Voltage>=405)&&(Voltage<410)) return 80;
    if ((Voltage>=400)&&(Voltage<405)) return 70;
    if ((Voltage>=395)&&(Voltage<400)) return 60;
    if ((Voltage>=390)&&(Voltage<395)) return 50;
    if ((Voltage>=385)&&(Voltage<390)) return 40;
    if ((Voltage>=380)&&(Voltage<385)) return 30;
    if ((Voltage>=370)&&(Voltage<380)) return 20;
    if ((Voltage>=360)&&(Voltage<370)) return 10;
    return 5;
}



void COM_Send(char c)
{
    UDR0=c;
    while(1)
    {
        if(UCSR0A & 0b00100000) break;    
    }
}

interrupt [TIM0_OVF] void timer0_ovf_isr(void)
{
   time0++;
   if (time0>20)
   {
        btn_push=0;  
        delayADC=0;
        alarm_count++;
        delay_count++;
                
        time0=0;
   }
}

interrupt [TIM1_COMPA] void timer1_compa_isr(void)
{
// 
    time_sound++;
    //   
    //   0.8 . ..      40  ( 0.02)
    if ((time_sound>40)&&(PlayOn)&&(delay_count>delay_sound_start))
    {
        if (Byte2Play[ByteCountTimer]>0) COM_Send(Byte2Play[ByteCountTimer]);
        ByteCountTimer++;
        time_sound=0;    
        if (ByteCountTimer>ByteCount)
        {
            PlayOn=0;
            ByteCount=0; 
            Byte2Play[0]=0;
            Byte2Play[1]=0;
            Byte2Play[2]=0;
            Byte2Play[3]=0;
            ByteCountTimer=0; 
           
        }                          
    }
}



// Read the AD conversion result
unsigned int ADC_result()
{
delay_us(10);
// Start the AD conversion
ADCSRA |= 0x40;
while(ADCSRA & 0x40) {}
delayADC=1; //   
return ADCW;
}

interrupt [USART_RXC] void usart_rx_isr(void)
{
    char Pos=ReceivePos1;
    char NextPos=(Pos+1)&31;
    if(NextPos!=ReceivePos0)
    {
        ReceiveBuffer[Pos]=UDR0;
        ReceivePos1=NextPos;
    }
}

char COM_Receive() // Get byte from receive buffer
{
    char Data=ReceiveBuffer[ReceivePos0];
    ReceivePos0=(ReceivePos0+1)&31;       
    return Data;
}


void Button()
{
    time0=0;
    PlayOn=0;
    PlayOn=0;
    ByteCount=0; 
    ByteCountTimer=0; 
    Byte2Play[0]=0;
    Byte2Play[1]=0;
    Byte2Play[2]=0;
    Byte2Play[3]=0; 
    disp++;
    count_metr=5;
    if (disp>4) disp=1; 
    time_sound=0; // ,       .  
    if (disp==1) {Led1=1;Led2=0;Led3=0;COM_Send(s_onboard_volt);};
    if (disp==2) {Led1=0;Led2=1;Led3=0;COM_Send(s_lipo);};
    if (disp==3) {Led1=0;Led2=0;Led3=1;if (BAT>107) COM_Send(s_bat); else COM_Send(s_bat_low);};
    if (disp==4) {Led1=1;Led2=1;Led3=1;Byte2Play[0]=s_alt;ByteCount=1;PrevAltitude=9999;};
    btn_push=1; // ...    :(   

}

void Display(void)
{
int DG=0;
int DG2=0;

     //   10  .    1  2 
      if (disp==1) DG=AV1*0.5082;    //     ( )
      if (disp==2) DG=AV2*0.5082;    //     ( )
      if (disp==3) DG=BAT;           //    
      if (disp==4) DG=Altitude;      //   RX   FrSky.    -  . 
      
     // DG = ByteCountTimer;
      
      PORTB = (Symbol[DG/100]);
      Dig1 = 1;
      delay_us(LED_delay);
      Dig1 = 0;
      
      DG2=DG%100;
      PORTB = (Symbol[DG2/10]);
      if (disp!=4) PORTB.5=0;         //   -   .    -   
      Dig2 = 1;
      delay_us(LED_delay);
      Dig2 = 0;          
      

      PORTB = (Symbol[DG2%10]);
      Dig3 = 1;
      delay_us(LED_delay);
      Dig3 = 0;
         

};
 


void main(void)
{

bit b7E=0;     
bit bTelemetry=0;
bit bData=0;
char CountByte=0;


PORTC=0b00000010;
DDRC= 0b00000001;

PORTB=0b00000000;
DDRB= 0b11111111;

PORTD=0b00000001;
DDRD= 0b11111110;

Led1=1;Led2=0;Led3=0;

// USART initialization
// Communication Parameters: 8 Data, 1 Stop, No Parity
// USART Receiver: On
// USART Transmitter: On
// USART Mode: Asynchronous
// USART Baud Rate: 9600
UCSR0A=0x00;
UCSR0B=0x98;
UCSR0C=0x06;
UBRR0H=0x00;
UBRR0L=0x33;


TCCR0B=0x05;

TCCR1B=0x0D;  //Mode: CTC top=OCR1A
OCR1AH=0x00; //   0.02   
OCR1AL=0x9C;

TIMSK0=0x01;
TIMSK1=0x02;

Led1=1;

ADMUX=0b11000110;
ADCSRA=0b10000111;
ADCSRA |= 0b01000000;
while(ADCSRA & 0b01000000) {}


#asm("sei")

while (1)
{
    
      unsigned int  CurrentAlt=0;       
      unsigned char CurrentVolt=0; 
            
      Display();  
      
      if ((BAT<107)&&(alarm_count>50)&&(!PlayOn))    //       
      {
            time_sound=0; // ,       .
            COM_Send(s_bat_low);
            alarm_count=0;  
      } 
      
      //   -   ,     (    -        )
      //  -       -       ,    ..
      
      if (delayADC==0) BAT=ADC_result()*1.16;  //   
      
     
      if ((!Btn)&&(btn_push==0)&&((delay_count>delay_sound_start))) 
      {
            Button();
      }  
        
      while(COM_DataPresent())
      {
            char c=COM_Receive();
            if(c==0x7E) //  / 
            {
                if (b7E) CountByte=0;
                b7E=1; 
                bTelemetry=0;
                bData=0;
            }; 
            if(c==0xFE)//    
            {
                if (b7E) 
                {
                    bData=0;
                    bTelemetry=1;
                };  
                CountByte=2;  //   -              
            }; 
            if(c==0xFD)   //    
            {
                if (b7E) 
                {
                    bData=1;
                    bTelemetry=0;
                };  
                CountByte=2;  //   -              
            }; 
            if ((c!=0x7E)&&(c!=0xFE)&&(c!=0xFD)) 
            {
                b7E=0;
                if ((CountByte==4)&&(bData)&&(c!=0x1F)&&(c!=0xE6)) Altitude=c*4;  //  4 ( - 4 ).   . (124 , 920 )
                if ((CountByte==2)&&(bTelemetry)) AV1=c;
                if ((CountByte==3)&&(bTelemetry)) AV2=c;         
                CountByte++; 
            };  
            
            
            
      };


      //      UART (  ),                                                             
      //#asm("cli")

      if (!PlayOn)
      {
            CurrentAlt=Altitude;
            CurrentVolt=Volt2Persent(AV2*1.694); 
            
            if ((abs(CurrentVolt-PrevVoltage)>9)&&(disp==2)&&(CurrentVolt!=5))
            {
                Byte2Play[ByteCount]=(CurrentVolt/10)+64;       //    %.  - 10%.  -      *100
                ByteCount++;
                Byte2Play[ByteCount]=s_percent; // 
                ByteCount++; 
                ByteCountTimer=0;  
                PrevVoltage=CurrentVolt;
                PlayOn=1;
                time_sound=30;
            } 
            
           
            if ((abs(CurrentAlt-PrevAltitude)>9)&&(disp==4)) //       10 -   
            {
                if (CurrentAlt>99) 
                {               
                    Byte2Play[ByteCount]=((CurrentAlt/100)+73); // -   J.. K.. R.. (J - 10  ..)
                    ByteCount++;
                }    
                if (((CurrentAlt/10)%10)>0)
                {
                    Byte2Play[ByteCount]=((CurrentAlt/10)%10+64); // -    
                    ByteCount++;
                }
                if ((CurrentAlt==0)) 
                {
                    Byte2Play[ByteCount]=64;
                    ByteCount++;
                }
                if ((ByteCount>0)&&(count_metr>4))
                {
                    count_metr=0;
                    Byte2Play[ByteCount]=s_metr; //   
                    ByteCount++;
                }
                count_metr++; 
                ByteCountTimer=0;  
                PrevAltitude=Altitude;
                PlayOn=1;                     
                time_sound=30;
            }
      }
      //#asm("sei")
 
}
}
